@@ -5,7 +5,7 @@ use color_eyre::{
 use execute::{shell, Execute};
 use serde_json::json;
 
-use crate::{color::color::get_closest_color, parser::Engine as NewEngine};
+use crate::{color::color::get_closest_color, helpers::get_syntax, parser::Engine};
 use serde::{Deserialize, Serialize};
 
 use std::{collections::HashMap, path::Path, process::Stdio, str};
@@ -23,13 +23,24 @@ use crate::{SchemesEnum, State};
 #[derive(Serialize, Deserialize, Debug, Clone)]
 pub struct Template {
     pub input_path: PathBuf,
-    pub output_path: Option<PathBuf>,
+    pub output_path: Option<OutputPath>,
     pub mode: Option<SchemesEnum>,
     pub colors_to_compare: Option<Vec<crate::color::color::ColorDefinition>>,
     pub compare_to: Option<String>,
     pub pre_hook: Option<String>,
     pub post_hook: Option<String>,
     pub input_path_modes: Option<InputPathModes>,
+    pub expr_prefix: Option<String>,
+    pub expr_postfix: Option<String>,
+    pub block_prefix: Option<String>,
+    pub block_postfix: Option<String>,
+}
+
+#[derive(Serialize, Deserialize, Debug, Clone)]
+#[serde(untagged)]
+pub enum OutputPath {
+    Single(PathBuf),
+    Multiple(Vec<PathBuf>),
 }
 
 #[derive(Serialize, Deserialize, Debug, Clone)]
@@ -40,11 +51,11 @@ pub struct InputPathModes {
 
 pub struct TemplateFile<'a> {
     state: &'a State,
-    engine: &'a mut NewEngine,
+    engine: &'a mut Engine,
 }
 
 impl TemplateFile<'_> {
-    pub fn new<'a>(state: &'a State, engine: &'a mut NewEngine) -> TemplateFile<'a> {
+    pub fn new<'a>(state: &'a State, engine: &'a mut Engine) -> TemplateFile<'a> {
         TemplateFile { state, engine }
     }
 
@@ -54,8 +65,7 @@ impl TemplateFile<'_> {
             &self.state.config_file.templates.len()
         );
 
-        let mut paths_hashmap: HashMap<String, (PathBuf, Option<PathBuf>)> = HashMap::new();
-        let mut template_export_count = 0;
+        let mut paths_hashmap = HashMap::new();
 
         for (_i, (name, template)) in self.state.config_file.templates.iter().enumerate() {
             let input_path = if let Some(input_path_mode) = &template.input_path_modes {
@@ -67,30 +77,51 @@ impl TemplateFile<'_> {
                 &template.input_path
             };
 
-            let (input_path_absolute, output_path_absolute) =
+            let (input_path_absolute, output_paths_absolute) =
                 get_absolute_paths(&self.state.config_path, input_path, &template.output_path)?;
 
             if !input_path_absolute.exists() {
-                warn!("<d>The <yellow><b>{}</><d> template in <u>{}</><d> doesnt exist, skipping...</>", name, input_path_absolute.display());
+                warn!("<d>The <yellow><b>{}</><d> template in <u>{}</><d> doesn't exist, skipping...</>", name, input_path_absolute.display());
                 continue;
             }
 
-            if template.output_path.is_some() {
-                template_export_count += 1;
-            }
+            let old_syntax = match (
+                &template.block_prefix,
+                &template.block_postfix,
+                &template.expr_prefix,
+                &template.expr_postfix,
+            ) {
+                (None, None, None, None) => None,
+                _ => {
+                    let old_syntax = self.engine.set_syntax(get_syntax(
+                        template.block_prefix.as_ref(),
+                        template.block_postfix.as_ref(),
+                        template.expr_prefix.as_ref(),
+                        template.expr_postfix.as_ref(),
+                    ));
+                    Some(old_syntax)
+                }
+            };
 
             let data = read_to_string(&input_path_absolute)
                 .wrap_err(format!("Could not read the {} template.", name))
                 .suggestion("Try converting the file to use UTF-8 encoding.")?;
 
             self.engine.add_template(name.to_string(), data);
-            paths_hashmap.insert(
-                name.to_string(),
-                (input_path_absolute, output_path_absolute),
-            );
+
+            for output_path in output_paths_absolute {
+                paths_hashmap.insert(
+                    name.to_string(),
+                    (input_path_absolute.to_path_buf(), output_path),
+                );
+            }
+
+            if let Some(old) = old_syntax {
+                self.engine.set_syntax(old);
+            };
         }
 
-        for (i, (name, template)) in self.state.config_file.templates.iter().enumerate() {
+        for (name, template) in self.state.config_file.templates.iter() {
             if let Some(hook) = &template.pre_hook {
                 info!("Running pre_hook for the <b><cyan>{}</> template.", &name);
                 format_hook(
@@ -102,11 +133,11 @@ impl TemplateFile<'_> {
                 .wrap_err(format!("Failed to format the following hook:\n{}", hook))?;
             }
 
-            let (input_path_absolute, output_path_absolute) = paths_hashmap
-                .get(name)
-                .wrap_err("Failed to get the input and output paths from hashmap")?;
+            if template.output_path.is_some() {
+                let (input_path_absolute, output_path_absolute) = paths_hashmap
+                    .get(name)
+                    .wrap_err("Failed to get the input and output paths from hashmap")?;
 
-            if let Some(output_path_absolute) = output_path_absolute {
                 debug!(
                     "Trying to write the {} template from {} to {}",
                     name,
@@ -114,7 +145,7 @@ impl TemplateFile<'_> {
                     output_path_absolute.display()
                 );
 
-                self.export_template(name, output_path_absolute, i, template_export_count)?;
+                self.export_template(name, output_path_absolute)?;
             }
 
             if let Some(hook) = &template.post_hook {
@@ -128,32 +159,44 @@ impl TemplateFile<'_> {
                 .wrap_err(format!("Failed to format the following hook:\n{}", hook))?;
             }
         }
+
         Ok(())
     }
 
-    fn export_template(
-        &self,
-        name: &String,
-        output_path_absolute: &PathBuf,
-        i: usize,
-        template_count: i32,
-    ) -> Result<(), Report> {
+    fn export_template(&self, name: &String, output_path_absolute: &PathBuf) -> Result<(), Report> {
         let data = match self.engine.render(name) {
             Ok(v) => v,
             Err(errors) => {
                 for err in errors {
                     err.emit(&self.engine);
                 }
+
+                if self.state.args.continue_on_error.unwrap_or(false) {
+                    return Ok(());
+                }
+
                 std::process::exit(1);
             }
         };
 
         let out = if self.state.args.prefix.is_some() && !cfg!(windows) {
-            let mut prefix_path = PathBuf::from(self.state.args.prefix.as_ref().unwrap());
+            let mut prefix_path = PathBuf::from(
+                self.state
+                    .args
+                    .prefix
+                    .as_ref()
+                    .ok_or_else(|| Report::msg("Couldn't get the prefix path"))?,
+            );
 
-            let output_path = output_path_absolute
-                .strip_prefix("/")
-                .expect("output_path_absolute is not an absolute path.");
+            let output_path = match output_path_absolute.strip_prefix("/") {
+                Ok(v) => v,
+                Err(e) => {
+                    return Err(Report::msg(format!(
+                        "Output path is not an absolute path: {}",
+                        e
+                    )))
+                }
+            };
 
             prefix_path.push(output_path);
 
@@ -193,9 +236,7 @@ impl TemplateFile<'_> {
         output_file.write_all(data.as_bytes())?;
 
         success!(
-            "[{}/{}] Exported the <b><green>{}</> template to <d><u>{}</>",
-            i + 1,
-            &template_count,
+            "Exported the <b><green>{}</> template to <d><u>{}</>",
             name,
             output_path_absolute.display()
         );
@@ -205,7 +246,7 @@ impl TemplateFile<'_> {
 }
 
 fn format_hook(
-    engine: &mut NewEngine,
+    engine: &mut Engine,
     hook: &String,
     colors_to_compare: &Option<Vec<crate::color::color::ColorDefinition>>,
     compare_to: &Option<String>,
@@ -221,7 +262,7 @@ fn format_hook(
                 std::process::exit(1);
             }
         };
-        let closest_color = get_closest_color(compare, &res);
+        let closest_color = get_closest_color(compare, &res)?;
         engine.add_context(json!({
             "closest_color": closest_color
         }));
@@ -294,38 +335,54 @@ fn create_missing_folders(output_path_absolute: &Path) -> Result<(), Report> {
     Ok(())
 }
 
+pub fn get_absolute_path(base_path: &PathBuf, relative_path: &PathBuf) -> Result<PathBuf, Report> {
+    let base = std::fs::canonicalize(base_path)?;
+    let absolute = relative_path
+        .try_resolve_in(&base)?
+        .to_path_buf()
+        .strip_canonicalization();
+    Ok(absolute)
+}
+
 fn get_absolute_paths(
     config_path: &Option<PathBuf>,
     input_path: &PathBuf,
-    output_path: &Option<PathBuf>,
-) -> Result<(PathBuf, Option<PathBuf>), Report> {
-    let (input_path_absolute, output_path_absolute) = if config_path.is_some() {
-        let base = std::fs::canonicalize(config_path.as_ref().unwrap())?;
+    output_paths: &Option<OutputPath>,
+) -> Result<(PathBuf, Vec<PathBuf>), Report> {
+    let output_paths = match output_paths {
+        Some(OutputPath::Single(path)) => &Vec::from([path.to_path_buf()]),
+        Some(OutputPath::Multiple(paths)) => paths,
+        None => &Vec::new(),
+    };
+
+    let (input_path_absolute, output_paths_absolute) = if let Some(p) = config_path {
+        let base = std::fs::canonicalize(p)?;
+        let mut paths = Vec::new();
+
+        for output_path in output_paths {
+            paths.push(
+                output_path
+                    .try_resolve_in(&base)?
+                    .to_path_buf()
+                    .strip_canonicalization(),
+            );
+        }
+
         (
             input_path
                 .try_resolve_in(&base)?
                 .to_path_buf()
                 .strip_canonicalization(),
-            if let Some(output_path) = output_path {
-                Some(
-                    output_path
-                        .try_resolve_in(&base)?
-                        .to_path_buf()
-                        .strip_canonicalization(),
-                )
-            } else {
-                None
-            },
+            paths,
         )
     } else {
-        (
-            input_path.try_resolve()?.to_path_buf(),
-            if let Some(output_path) = output_path {
-                Some(output_path.try_resolve()?.to_path_buf())
-            } else {
-                None
-            },
-        )
+        let mut paths = Vec::new();
+
+        for output_path in output_paths {
+            paths.push(output_path.try_resolve()?.to_path_buf());
+        }
+
+        (input_path.try_resolve()?.to_path_buf(), paths)
     };
-    Ok((input_path_absolute, output_path_absolute))
+    Ok((input_path_absolute, output_paths_absolute))
 }
